#include "i_lib.h"
#include "i_mouse.h"
#include "standard.h"

/*
ͻ
 Routine      Author : David J. Yazel                                      
              Date :   October 9, 1989                                     
              Name :   I_mouse()                                           
Ķ
 Description  This routine will determine if there are any inputs          
              available for the mouse by calling the mouse driver and      
              asking it.                                                   
Ķ
 Returns      TRUE : There is mouse input to be had.                       
              FALSE : There is no mouse input available.                   
ͼ  
*/

boolean I_mouse()

begin

   return (MOUSE_event());

end

boolean I_mouse_get( input,see )

I_input_t *input;
boolean see;

begin

    MOUSE_event_type event;

    /* if there are mouse events then see what they are */

    if (MOUSE_event())
      begin

        /* get the event from the mouse driver */

        if (see) MOUSE_see_event( &event );
        else MOUSE_get_event( &event );

        /* look at the function returned by the event to determine what */
        /* to translate that to */

        if ((event.func&2)==2)        input->func = I_MPUSH_L;
        else if ((event.func&4)==4)   input->func = I_MRELEASE_L;
        else if ((event.func&8)==8)   input->func = I_MPUSH_R;
        else if ((event.func&16)==16) input->func = I_MRELEASE_R;
        else if ((event.func&1)==1)   input->func = I_MMOVE;

        /* record the position */

        input->x = event.x;
        input->y = event.y;
        input->clock = event.clock;

        return TRUE;


      end
    else return (FALSE);


end

static trash_mouse_movement()

begin


   MOUSE_event_type event;

   if (MOUSE_event())
   do begin

      MOUSE_see_event( &event );
      if ( event.func == 1 ) MOUSE_get_event( &event );

   end while ((event.func == 1) && (MOUSE_event()));

end

/*
ͻ
 Routine      Author : David J. Yazel                                      
              Date :   October 9, 1989                                     
              Name :   I_get_mouse()                                 
Ķ
 Description  This routine will talk to the mouse driver to determine      
              what mouse action has been done.  It may combine several     
              mouse actions together into one mouse input.                 
Ķ
 Parameters   input : The input event record to be filled in.              
ͼ  
*/

boolean I_get_mouse( input )

I_input_t *input;

begin

    #define CLICK_SENSITIVITY 7

    clock_t ticks;
    I_input_t next_input;

    /* if there are mouse events then see what they are */

    if (!I_mouse_get(input,FALSE)) return (FALSE);

    /* if the function is a press of either the right or left button then */
    /* we need to determine if there is a release of the button quickly */
    /* enough to constitute a CLICK of the mouse button. */

    if ((input->func==I_MPUSH_R) || (input->func==I_MPUSH_L))
      begin

        /* if there is not currently a mouse event, then determine if */
        /* enough time has passed to indicate that a release will not */
        /* be there. */

        trash_mouse_movement();
        if (!MOUSE_event())
          begin

            /* get the current number of ticks.  If the number of ticks */
            /* is signifigantly bigger than the last mouse event, it can */
            /* be assumed that the mouse press is not a click */

            ticks = clock();
            if (ticks-input->clock>CLICK_SENSITIVITY) return (TRUE);

            /* if not enough time has passed to determine if it was */
            /* a click or not, wait a short while. if there is still no */
            /* event then exit */

            do begin

               trash_mouse_movement();
               if (I_mouse()) break;

            end while ((clock()-input->clock)<CLICK_SENSITIVITY);

            /* check for input from mouse */

            trash_mouse_movement();
            if (!I_mouse()) return (TRUE);

          end

        /* now check to see if the mouse option is a release of the */
        /* button which was pressed */

        I_mouse_get( &next_input, TRUE );

        /* check and see if the mouse action is the release of the button */
        /* pushed */
        
        if ((input->func==I_MPUSH_L) && 
           (next_input.func==I_MRELEASE_L)) 
          begin
            input->func = I_MCLICK_L;
            I_mouse_get( &next_input, FALSE );
          end

        /* check and see if the mouse action is the release of the button */
        /* pushed */
        
        if ((input->func==I_MPUSH_R) && 
           (next_input.func==I_MRELEASE_R)) 
          begin
            input->func = I_MCLICK_R;
            I_mouse_get( &next_input, FALSE );
          end

      end

    /* return TRUE because no problems were found */

    return (TRUE);

end


/*
ͻ
 Routine      Author : David J. Yazel                                      
              Date :   October 9, 1989                                     
              Name :   I_input()                                           
Ķ
 Description  This boolean function checks if there is any input at all    
              from the keyboard and the mouse.                             
ͼ  
*/

boolean I_input()

begin

   return (keypressed() || I_mouse());

end

/*
ͻ
 Routine      Author : David J. Yazel                                      
              Date :   October 9, 1989                                     
              Name :   I_get()                                       
Ķ
 Description  This routine will return the current input from the mouse or 
              the keyboard. If there is none it will wait until there is.  
Ķ
 Parameters   input : The input record to fill in with the information.    
Ķ
 Returns                                                                   
                                                                           
                                                                           
                                                                           
ͼ  
*/

void I_get( input )

I_input_t *input;

begin


    /* loop forever until there is a mouse function or a key hit */

    do begin

       /* check for a mouse input */

       if (I_mouse())
         begin

           I_get_mouse(input);
           break;

         end

       /* check to see if there is a keyboard input.  If so then */
       /* read the key and return a KKEYHIT function */

       if (keypressed())
         begin

           input->func = I_KKEYHIT;
           input->key = readkey();
           break;

         end

    end while (TRUE);

end


